# -*- coding: utf-8 -*-

"""This module contains generators and maybe later readers for some
common fileformats"""

from time import strftime

class RSS:
    """This is only a basic RSS 2.0, not all extension above the most
    basic are supported"""
    RSS_BODY = """<?xml version="1.0"?>
<rss version="2.0">
<channel>
    <title>%s</title>
    <link>%s</link>
    <description>%s</description>
%s
</channel>
</rss>"""

    RSS_ITEM = """
    <item>
      <title>%s</title>
      <link>%s</link>%s
      <description>%s</description>
    </item>
"""

    def __init__( self, title, link = "http://blambi.hopto.org:8080/kolli/",
                  description = "" ):
        self.title = title
        self.link = link
        self.description = description
        self.items = list()

    def add_item( self, title, link = "", description = "", timestamp = False ):
        """Add an item to the rss feed, timestamp should be a
        timetuple if used"""
        if not timestamp:
            str_time = ""
        else:
            str_time = "\n      <pubDate>%s</pubDate>" %(
                strftime( "%a, %d %b %Y %H:%M:%S CEST", timestamp ) )
        
        if self.items == []:
            self.items = [ self.RSS_ITEM %( title, link, str_time,
                                            description ) ]
        else:
            self.items.append( self.RSS_ITEM %( title, link, str_time,
                                                description ) )

    def output( self ):
        """Returns the XML for the RSS, and adds empty items as needed"""
        if self.items == []:
            items = self.RSS_ITEM %( "", "", "", "" ) 
        else:
            items = ""
            for item in self.items:
                items += item

        return self.RSS_BODY %( self.title,  self.link, self.description,
                                items )

if __name__ == '__main__':
    print "Testing"
    rss = RSS( "Test" )
    rss.output()
    print "Empty RSS ok"
