# -*- coding: utf-8 -*-

"""Currently an access to Boardgamegeeks.com's XMLAPI and Kakwiki
Simple Interface.

There is also an little helper for translating BGG.get_wishlist's key
wishlist_level to a string called wishlist_helper"""

import urllib, urllib2
import xml.dom.minidom as minidom
import hashlib, base64, hmac
from blamlib.helpers import xml_get_text

class BGG:
    """simple XMPAPI access for boardgamegeeks.com"""
    def __init__( self, username ):
        self.username = username
        self.XML_API = "http://www.boardgamegeek.com/xmlapi/collection/%s"
        
    def get_xml( self ):
        """Downloads the XML and returns it AND keeps an internal copy."""
        self.xmldata = minidom.parseString(
            urllib.urlopen( self.XML_API % self.username ).read() )
        return self.xmldata

    def get_owned( self, rated = True ):
        """Returns a list of dicts for all games. rated = Just returns
        the rated games, False just the unrated

        Dict description:
        url = url for the game
        bgg_rating = bgg rating, a value based on all who have rated this game
        rating = your rating, if any (else its None)
        name = name of the game
        """

        games = self.xmldata.getElementsByTagName( 'item' )

        if rated:
            games = filter( lambda item: int(
                    item.getElementsByTagName( 'status' )[0].\
                        getAttribute( 'own' ) ) and \
                                item.getElementsByTagName( 'stats' )[0].\
                                getAttribute( 'rating' ) != "", games )

            games.sort( lambda x, y: int(
                    y.getElementsByTagName( 'stats' )[0].\
                        getAttribute( 'rating' ) ) - int(
                    x.getElementsByTagName( 'stats' )[0].\
                        getAttribute( 'rating' ) ) )
            
        else:
            # Really don't want to be part of better formating :(
            games = filter( lambda item: int( item.getElementsByTagName( 'status' )[0].getAttribute( 'own' ) ) and item.getElementsByTagName( 'stats' )[0].getAttribute( 'rating' ) == "", games )

        retval = list()

        for item in games:
            game = dict()
            game['name'] = item.getElementsByTagName( 'name' )[0].\
                childNodes[0].nodeValue
            game['url'] = "http://boardgamegeek.com/boardgame/" + \
                item.getAttribute( 'objectid' )
            
            x = item.getElementsByTagName( 'stats' )[0].\
                getAttribute( 'bggrating' )
            if x != "":
                game['bgg_rating'] = int( float( x ) )
            else:
                game['bgg_rating'] = None

            if rated:
                game['rating'] = item.getElementsByTagName( 'stats' )[0].\
                    getAttribute( 'rating' )
            else:
                game['rating'] = None

            retval.append( game )

        return retval
                
    def get_wanted( self ):
        """Returns all games on the wishlist, similar dict to
        get_owned but got an ordered and wishlist-level keys instead
        of a rating key."""
        
        games = self.xmldata.getElementsByTagName( 'item' )

        # Another one that wont look nice...
        games = filter( lambda item: int( item.getElementsByTagName( 'status' )[0].getAttribute( 'wishlist' ) ) and int( item.getElementsByTagName( 'status' )[0].getAttribute( 'wishlistpriority' ) ) < 5, games )

        games.sort( lambda x, y:
                        int( x.getElementsByTagName( 'status' )[0].\
                                 getAttribute( 'wishlistpriority' ) ) -\
                        int( y.getElementsByTagName( 'status' )[0].\
                                 getAttribute( 'wishlistpriority' ) ) )

        retval = list()

        for item in games:
            game = dict()

            game['name'] = item.getElementsByTagName( 'name' )[0].\
                childNodes[0].nodeValue
            game['url'] = "http://boardgamegeek.com/boardgame/" + \
                item.getAttribute( 'objectid' )

            bgg_rating = item.getElementsByTagName( 'stats' )[0].\
                getAttribute( 'bggrating' )
            
            if bgg_rating != "":
                game['bgg_rating'] = int( float( bgg_rating ) )
            else:
                game['bgg_rating'] = None

            game['ordered'] = bool( int(
                    item.getElementsByTagName( 'status' )[0].\
                        getAttribute( 'preordered' ) ) )

            game['wishlist_level'] = int(
                item.getElementsByTagName( 'status' )[0].\
                    getAttribute( 'wishlistpriority' ) )

            retval.append( game )

        return retval

# useful for more readably wishlist values.
wishlist_helper = { 1: "Must have", 2: "Love to have", 3: "Like to have",
                    4: "Thinking about it", 5: "Don't want nooo!" }

class Kakwiki:
    """A simple interface client for kakwiki"""

    def __init__( self, base_url = "http://kakor.chebab.com" ):
        """base_url is for example http://kakor.chebab.com"""
        if not base_url.endswith( '/' ):
            base_url += '/'

        self.base_url = base_url
        self.authed = False
        self.cookie = None
        
    def read( self, page = "Portal" ):
        """Returns the raw wikitext for specified page"""
        if not self.authed:
            wiki_file = urllib.urlopen( self.base_url + "simp.php?read=%s" %
                                        page )
        else:
            wiki_file = urllib.urlopen( self.base_url +
                                        "simp.php?read=%s&cookie=%s"
                                        %( page, self.cookie ) )
        data = wiki_file.read()
        wiki_file.close()

        if data == "NONE":
            return None
        
        return data

    def list( self ):
        """Returns a dict with name as key and a tuple ( size,
        modification time ) as value both as int ( char, unix-time )"""
        ret = dict()

        wiki_file = urllib.urlopen( self.base_url + "simp.php?list" )

        for page in wiki_file.readlines():
            # name/size/mtime
            name, size, mtime = page.split( '/' )
            ret[name] = ( int( size ), int( mtime.strip() ) )

        return ret

    def auth( self, username, password ):
        """logins and gives a simple 'cookie' that will be used for writing"""
        wiki_file = urllib.urlopen( self.base_url + "simp.php?auth=%s&pass=%s"
                                    %( username, password ) )
        data = wiki_file.read()

        if data == '':
            return False

        self.authed = True
        self.cookie = data
        return True
    
    # TODOs
    def write( self, page, data ):
        """Writes data to specified page, but not supported yet and
        you must be logged in first"""
        if self.authed:
            print "WARNING: (web.Kakwiki.write) Not done!"
            return False
        else:
            return False


class Twitter:
    """Just a implemention of the twitter API"""
    api_url = "http://twitter.com/"

    class TwitterError( TypeError ):
        pass
    
    def __init__( self, user, password ):
        self.user = user
        self.password = password

        # This is confusing
        self._pass_mang = urllib2.HTTPPasswordMgrWithDefaultRealm()
        self._pass_mang.add_password( None, self.api_url, self.user,
                                         self.password )

        self._auth_handler = urllib2.HTTPBasicAuthHandler( self._pass_mang )

        self._opener = urllib2.build_opener( self._auth_handler )

    def __raw_query__( self, command, post_data = None ):
        # this is a low level function that communicates with the webserver.
        # Will return a string on success.
        urllib2.install_opener( self._opener ) # backup before?

        result = None
        
        try:
            if post_data != None:
                result = urllib2.urlopen( self.api_url + command,
                                          data = post_data ).read()
            else:
                result = urllib2.urlopen( self.api_url + command ).read()

        except urllib2.HTTPError, why:
            if why.code == 404:
                raise self.TwitterError, "Can't access: %s" %( self.api_url +
                                                               command )
            elif why.code == 401: # Unauthorized
                raise self.TwitterError, "Error with credentials"
            
            else:
                raise self.TwitterError, why # just a rebrand...

        return result

    def update( self, message ):
        """Add a new message to your tweet"""
        self.__raw_query__( "statuses/update.xml",
                            post_data = 'status=%s' % message )
        return True

    def get_friends_ids( self, user_id = None, screen_name = None ):
        """Returns a list of ids for either user_id or screen_names account"""
        options = str()
        
        if user_id:
            options = "?user_id=%d" % int( user_id )
        elif screen_name:
            options = "?screen_name=%s" % screen_name
        
        id_xml = self.__raw_query__( "friends/ids.xml" + options )

        values = self._get_ids( minidom.parseString( id_xml ) )
        return values

    def _get_ids( self, xml ):
        # Extracts all id's outof <ids><id/>..<id/></ids>
        return map( lambda node: node.firstChild != None and
                    node.firstChild.nodeValue,
                    xml.getElementsByTagName( 'ids' )[0].
                    getElementsByTagName( 'id' ) )

    def get_followers_ids( self, user_id = None, screen_name = None ):
        """Returns a list of ids for either user_id or screen_names account"""
        options = str()
        
        if user_id:
            options = "?user_id=%d" % int( user_id )
        elif screen_name:
            options = "?screen_name=%s" % screen_name

        id_xml = self.__raw_query__( "followers/ids.xml" + options )

        values = self._get_ids( minidom.parseString( id_xml ) )
        return values

    def show_user( self, user_id = None, screen_name = None ):
        """Returns a dict."""
        # FIXME: add more values in the return
        options = str()
        
        if user_id:
            options = "?user_id=%d" % int( user_id )
        elif screen_name:
            options = "?screen_name=%s" % screen_name
        else:
            return None # FIXME: raise an error.

        xml_data = minidom.parseString(
            self.__raw_query__( "users/show.xml" + options )
            )

        xml_data = xml_data.getElementsByTagName( 'user' )[0]
        
        data = dict()

        data['id'] = int( xml_get_text( xml_data, 'id' ) )
        data['name'] = xml_get_text( xml_data, 'name' )
        data['screen_name'] = xml_get_text( xml_data, 'screen_name' )
        data['description'] = xml_get_text( xml_data, 'description' )
        data['url'] = xml_get_text( xml_data, 'url' )
        data['location'] = xml_get_text( xml_data, 'location' )
        data['status'] = self._parse_status( xml_data )[0] # Only one..
        return data

    def verify_credentials( self ):
        """Checks that your username and password is correct, this
        version doesn't show the full profile etc just returns true or
        false."""

        try:
            xml_data = self.__raw_query__( "account/verify_credentials.xml" )
        except self.TwitterError, why:
            if str( why ) == "Error with credentials": # ungly..
                return False

            raise self.TwitterError, why

        return True
    
    def _parse_status( self, xml ):
        """Used for reading out one or more statuses as one or more dict's in a list"""
        statuses = xml.getElementsByTagName( 'status' )

        data = list()

        for status in statuses:
            st_data = dict()
            data.append( st_data )
            st_data['id'] = int( xml_get_text( status, 'id' ) )
            st_data['created_at'] = xml_get_text( status, 'created_at' )
            st_data['text'] = xml_get_text( status, 'text' )
            st_data['source'] = xml_get_text( status, 'source' )

        return data


class Identica( Twitter ):
    """A simple way to use the twitter compatible API for identi.ca"""
    api_url = "https://identi.ca/api/"

    # Oddities
    # get followers ids and get friends ids also list the user it self..
    # this doesn't happen against Twitter

    def get_version( self ):
        """Returns the server version"""
        xml_data = minidom.parseString(
            self.__raw_query__( "statusnet/version.xml" ) )

        return xml_get_text( xml_data, 'version' )


class Yubikey:
    """Verification of yubikey one-time-password"""

    class YubiError( TypeError ): pass
    class YubiRemoteError( YubiError ): pass
    
    def __init__( self, api_id, api_key,
                  server = "https://api.yubico.com/wsapi/" ):
        if len( api_key ) != 28:
            raise self.YubiError, 'Malformed API Key!'
        
        self.api = { 'id': api_id, 'key': base64.b64decode( api_key ) }
        self.server = server

    def __remote_verify__( self, otp ):
        """Checks if the server thinks this is a good otp"""
        ret = dict()
        yk_data = urllib.urlopen( self.server + "verify?id=%d&otp=%s" %
                                  ( self.api['id'], otp ) ).readlines()
        
        for line in yk_data:
            if line.find( '=' ) != -1:
                key, value = line.strip().split( '=', 1 )
                ret[key] = value

        # not malformed?
        if not ret.has_key( 'h' ) or not ret.has_key( 'h' ) \
                 or not ret.has_key( 'status' ):
            raise self.YubiRemoteError, "Got malformed data from the server"
        
        # raise the alarm if remote did not like us
        if ret['status'] != 'OK':
            raise self.YubiRemoteError, ret['status']
                
        return ret
               
    def verify( self, otp ):
        """Checks if a OTP is valid"""

        # simple protection against obvious invalids
        if len( otp ) != 44:
            raise self.YubiError, "Invalid amount of characters."

        # remote verify
        rv_data = self.__remote_verify__( otp )

        # check signature
        local_sig = ""
        for key, value in rv_data.items(): # is alphabetical so no pain here
            if key != 'h':
                local_sig += "%s=%s&" %( key, value )
               
        signer = hmac.new( self.api['key'], local_sig[:-1], hashlib.sha1 )
        local_sig = base64.urlsafe_b64encode( signer.digest() )

        if local_sig != rv_data['h']:
            raise YubiError, "Signature mismatch"

        # else just return the identification part so the program can
        # handle local on it's own.
        return True
